# Installation et configuration de Gitea

Cette étape couvre le processus entier de l'installation du service **Gitea**. 

Il s'agit d'une *forge logicielle* libre et open source écrite en langage [Go](https://go.dev), pour l'hébergement de répos Git, intégrant un système de suivi des bugs, un outil de création de wiki, ainsi qu'un set d'outils pour la relecture de code. *([About - Gitea](https://about.gitea.com))*

Il présente de très nombreuses similarités avec des services d'hébergement git plus connus, comme [GitHub](https://github.com) ou encore [GitLab](https://gitlab.com), avec l'avantage d'être complètement open-source, et totalement hébergeable, sans license (comme c'est le cas pour GitLab). 

Le logiciel a vu le jour le 17 octobre 2016, à la suite du fork de [Gogs](https://gogs.io), qui est par ailleurs encore activement maintenu. Le code de ce dernier a donc été copié et réutilisé dans le projet Gitea.

Voyons donc ici comment l'installer sur notre machine virtuelle.

## Redirection de ports sur la machine hôte

Afin de pouvoir utiliser le service Gitea — puisque la machine invitée ne possède pas d'interface graphique -, il sera nécessaire d'ouvrir au moins deux ports : 
 - Le port 220 -> 22 : qui permettra d'accéder à la machine virtuelle en SSH, pour gagner du temps lors de la copie des commandes de la documentation, mais également d'accéder au code source stocké sur le serveur par SSH (qui est un des deux protocoles, avec HTTP, utilisés par Git)
 - Le port 3000 -> 3000 : qui sera utilisé pour afficher l'interface web de Gitea une fois le service installé. 

Étant donné que la machine est configurée en NAT (Network Adress Translation), le réseau n'est pas directement accessible depuis la machine, mais utilise un réseau interne fourni par VirtualBox, qui fait également office de pare-feu, **empêchant ainsi tout paquet de passer de l'hôte à la machine virtuelle**. 

VirtualBox permet cependant de configurer de la redirection de ports, qui permet de fournir un accès **partiel** à la machine distante, sur quelques ports.

### Récupération de l'adresse de la machine

La première étape consiste à récupérer l'adresse IP interne de la machine, afin de pouvoir la renseigner plus tard dans la table de redirection de port.

On démarre donc la machine, pour pouvoir taper la commande : 
```bash
ip a
```

On peut ainsi la garder en mémoire pour la suite et éteindre la machine pour le reste de l'opération.

### Configuration dans VirtualBox

La suite consiste à configurer la redirection de ports dans VirtualBox.

Pour cela, il faut donc aller dans les **paramètres réseau de la machine virtuelle** sur VirtualBox (Paramètres > Réseau), puis cliquer sur le bouton *"Redirection de ports"*. 

Une boîte de dialogue apparaît, et on peut alors créer deux nouvelle entrées pour la redirection de ports : 

| Nom      | Protocole | IP hôte   | Port hôte | IP invité | Port invité |
| -------- | --------- | --------- | --------- | --------- | ----------- |
| Gitea    | TCP       | 127.0.0.1 | 3000      | 10.0.2.15 | 3000        |
| SSH      | TCP       | 127.0.0.1 | 220       | 10.0.2.15 | 22          |


![Configuration réseau de la machine](assets/vbox-network-config.png)

*(Source : [IT-Connect.fr](https://www.it-connect.fr/configurer-le-port-forwarding-sur-une-vm-virtualbox%EF%BB%BF/))*

## Installation de docker

Pour faciliter l'installation de Gitea, on préfère tout faire tourner à l'intérieur d'un conteneur *Docker*. 

Cela a plusieurs avantages, notamment :
 - Une plus grande réparabilité en cas de problème, puisque aucun dommage n'est fait au système (tout est isolé à l'intérieur d'un environnement virtuel mais pas virtualisé)
 - Une meilleure sécurité, car le conteneur est concu par les développeurs de Gitea, et est donc étudié, ce qui élude les éventuelles failles dues à notre inattention

> **Docker**  
> Docker est un logiciel Français (cocorico !) qui se présente comme une "plateforme open-source permettant le développement, l'envoi et l'exécution d'applications". Docker permet surtout la séparation des applications et de l'infrastructure, ce qui rend la distribution du logiciel plus rapide et efficace.  
> Le logiciel a été crée en 2010 en tant que startup de la Y Combinator de Google.  
> *([What is Docker?, site officiel de Docker](https://docs.docker.com/get-started/docker-overview/), [Au Revoir, Simon Hykes, Docker](https://archive.ph/20210112150959/https://www.docker.com/blog/au-revoir/))*

Heureusement, il existe un moyen relativement simple d'installer Docker sur Debian, et toutes les instructions sont fournies par la [documentation officielle de Docker](https://docs.docker.com/engine/install/debian/)

Nous allons nous intéresser à l'installation APT, qui permet de synchroniser le fonctionnement de Docker et son installation avec APT. Ainsi, les mises à jour fournies par Docker seront automatiquement installées lors de la msie à jour globale du système. 

On commence donc par ajouter le dépôt Docker et ses clés de chiffrement aux sources `apt`: 
```bash 
# Installation des prérequis 
apt update
apt install ca-certificates curl
chmod a+r /etc/apt/keyrings/docker.asc

# Installation des clés de chiffrement
install -m 0755 -d /etc/apt/keyrings
curl -fsSL https://download.docker.com/linux/debian/gpg -o /etc/apt/keyrings/docker.asc

# Ajout du dépôt Docker aux sources d'APT
echo \
  "deb [arch=$(dpkg --print-architecture) signed-by=/etc/apt/keyrings/docker.asc] https://download.docker.com/linux/debian \
  $(. /etc/os-release && echo "$VERSION_CODENAME") stable" > etc/apt/sources.list.d/docker.list > /dev/null

# Synchronisation avec les dépôts Docker
apt update
```

Les sources de docker sont ainsi configurées, et nous sommes prêts pour l'installation : 

```bash
apt install docker-ce docker-ce-cli containerd.io docker-buildx-plugin docker-compose docker-compose-plugin
```

Une fois Docker installé, on peut donc passer à l'installation de Gitea.

## Installation de Gitea

L'installation de Gitea par docker fonctionne en utilisant un outil nommé `docker-compose`. Il permet de configurer un conteneur automatiquement en renseignant toutes ses spécifications via un fichier `.yml`. On peut donc aisément y ajouter ou personnaliser des fonctionnalités en fonction de ce que l'on souhaite obtenir.

Il suffit de créer un répertoire temporaire `gitea` : 
```bash
mkdir gitea
```

Puis d'y écrire un fichier `docker-compose.yml` : 
```yml 
# gitea/docker-compose.yml
version: "3"

networks:
  gitea:
    external: false

services:
  server:
    image: docker.gitea.com/gitea:1.23.4
    container_name: gitea
    environment:
      - USER_UID=1000
      - USER_GID=1000
    restart: always
    networks:
      - gitea
    volumes:
      - ./gitea:/data
      - /etc/timezone:/etc/timezone:ro
      - /etc/localtime:/etc/localtime:ro
    ports:
      - "3000:3000"
      - "222:22"
```

Ce fichier va au plus simple. Il s'assure que toutes les variables d'environnement sont configurées, que les volumes virtuels contenant les données de gitea sont bien montés, et que les ports sont bien configurés.

On peut donc ensuite lancer l'installation de Gitea via la commande : 
```bash
docker-compose up -d 
```

Et c'est parti. Normalement, le serveur démarre, et on peut accéder aux étapes d'installation de Gitea en utilisant l'interface web fournie. 

On y accède donc sur la machine hôte via l'adresse (si tout s'est bien passé) :
```
http://127.0.0.1:3000
```

*(Source : [Documentation officielle de Gitea](https://docs.gitea.com/installation/install-with-docker))*


Une fois tout ceci configuré, le serveur est prêt. On peut supprimer les dossiers temporaires, car Gitea est correctement installé et démarre tout seul avec la machine virtuelle.

![Configuration initiale de Gitea (matteyeux)](assets/gitea-install.PNG)

## Mise à jour de Gitea 

Pour mettre à jour Gitea par un conteneur Docker, il suffit de suivre la [documentation officielle de Gitea](https://docs.gitea.com/installation/install-with-docker)

Si la version de Gitea n'est pas directement spécifiée dans le fichier `docker-compose.yml`, on peut simplement lancer ces deux commandes pour automatiquement récupérer le dernier conteneur et remplacer l'ancien : 
```bash
docker-compose pull
docker-compose up -d
```

Et tout est mis à jour automatiquement. 

# Tests d'utilisation

Il est maintenant temps de tester notre installation en essayant d'utiliser le service Gitea disponible sur le port 3000.

Commençons donc par créer un nouvel utilisateur à partir de l'interface.

![Page d'accueil de Gitea](assets/gitea-home.png)

![Inscription d'un nouvel utilisateur](assets/gitea-register.png)

On arrive donc dans notre page de profil, où il n'existe pour l'instant aucun dépôt. Crééons-en donc un !

![Page de profil vierge Gitea](assets/gitea-profile.png)

Créons donc un nouveau dépôt pour y déposer la source des rapports de cette SAÉ.

On commence donc par cliquer sur + > Nouveau dépôt. On entre le nom du dépôt (ici, ce sera les rapports pour S2.03), une description si besoin, puis on clique sur "Créer" pour finaliser la création du dépôt. Exactement comme sur Gitlab ou GitHub.

![Nouveau dépôt sur Gitea](assets/gitea-new-repo.png)

On arrive ainsi sur une nouvelle page indiquant que le nouveau dépôt est crée.

On souhaite maintenant y ajouter du code. Il est l'heure de **manipuler un peu le terminal de la machine hôte**.

Comme le dépôt local existe déjà, et possède déjà une **remote**, qui pointe vers le serveur Gitlab de l'université, il faudra simplement en ajouter une nouvelle

```bash
git remote add gitea http://10.21.215.74:3000/robert/S2.03
``` 
Une fois cela fait, on peut simplement envoyer les modifications sur gitea.

```
git push gitea main
```

L'argument `gitea` indique que l'on souhaite bel et bien envoyer le code sur la remote `gitea`, que l'on vient juste de créer.

Git nous demande d'entrere un nom d'utilisateur ainsi qu'un mot de passe pour l'authentification, et une fois cela entré, il suffit d'attendre et de valider l'envoi des changements sur le serveur distant. 

Une fois cela fait, on met simplement à jour la page, et on a cette fois-ci accès au code complet de la SAÉ.

![Le dépôt S2.03 une fois les changements envoyés](assets/gitea-full-repo.png)

Tout ceci semble fonctionner plutôt bien. On peut donc affirmer que le serveur Gitea est opérationnel et fonctionne bien.

# CI/CD, Actions : Pour aller plus loin

Les hébergeurs de code Git intègrent souvent des outils d'intégration et déploiement en continu (CI/CD pour Continuous Integration/Continuous Deployment). 

## Définitions et principes

L'**intégration continue** consiste à automatiquement intégrer des modifications de code dans un dépôt partagé (ex: dépôt git), et à compiler et tester automatiquement le logiciel pour s'assurer qu'il continue de fonctionner comme prévu.

Le **déploiement continu**, dans le même temps, est la pratique consistant à déployer (compiler et envoyer en *release*) automatiquement de nouvelles versions du projet une fois qu'elles ont **passé les étapes de construction et de test** (intégration). Cela permet de publier plus rapidement les nouvelles fonctionnalités, pour **éviter de devoir tout faire manuellement**.

### Pipeline CI/CD

L'intégration continue et le déploiement continu réunis s'appellement **le pipeline CI/CD**. Il s'agit d'une combinaison d'outils (au sens informatique du terme, donc des *commandes*), de pratiques et de processus qui **fonctionnent ensemble pour intégrer, tester, et déployer en continu les changements de code**.

Sa mise en oeuvre passe souvent par plusieurs pratiques : 
 - Compilation automatique de code dans des environnements contrôlés (machines virtuelles)
 - Tests du projet, donc à la fois des tests unitaires, et fonctionnels (exécution surveillée du projet dans des conteneurs automatisés pour vérifier son bon fonctionnement)
 - Publication automatisée des nouvelles versions sur le serveur de distribution.

La raison pour laquelle les plateformes de gestion de code intègrent souvent ces ensembles d'outils est leur **grande omnipotence**. En effet, la plupart de ces plateformes réunissent :
 - Un dépôt partagé de code
 - Une documentation éditable (wiki)
 - Un système de distribution de versions fixes (tags, releases)
 - Pour certains, un serveur web (GitHub Pages, Gitlab Pages).
Il est donc parfaitement logique que ces plateformes mettent à disposition un système permettant de **relier cet ensemble d'outils via du scripting**.

## Utilisation sur Gitea

Fort heureusement, Gitea possède tout ce qu'il faut, par le biais de ses *Actions* (comme sur GitHub), pour faire fonctionner des pipelines basiques de CI/CD.

### Installation du *runner*

La première étape consiste à installer un *runner*. Il s'agit d'un programme ayant pour but d'exécuter des commandes, afin de mener à bien des tâches automatisées. 

#### Téléchargement de `act_runner`

Il faut donc tout d'abord récupérer un `act_runner` depuis la [page des releases officielle](https://gitea.com/gitea/act_runner/releases). Il existe une bonne quantité de choix de binaires, suivant le système d'exploitation et l'architecture du processeur de la machine hôte. Nous choisirons ici `act_runner-linux-arm64`. 

L'idée est de placer ce programme dans un endroit où le service Gitea pourra y avoir accès. Il servira à exécuter toutes les commandes des pipelines CI/CD.

Le "home directory" `~` de l'utilisateur `git` fera parfaitement l'affaire. 
```bash
# Connection en tant que git
su - git
cd ~

# Téléchargement de l'act_runner
wget https://gitea.com/gitea/act_runner/releases/download/v0.2.11/act_runner-0.2.11-linux-arm64
mv act_runner-0.2.11-linux-arm64 act_runner

# Ajout des permissions exécutable et vérifications
chmod +x act_runner 
./act_runner --version
```
#### Enregistrement du programme 

Pour permettre à Gitea d'utiliser `act_runner`, il faut tout d'abord **récupérer un token d'enregistrement**. 

Connectons-nous à notre utilisateur Gitea (le premier crée, l'administrateur), et rendons-nous à cette adresse :   
> `https://localhost:3000/-/admin/actions/runners`

On a alors accès à la page d'administration des actions, depuis laquelle on peut obtenir une liste de runners enregistrés.

![Administration - runners Gitea](assets/admin-runners.png)

Pour en créer un, il faut lui passer un **token d'enregistrement**, qu'on vient récupérer ici.

En haut à droite, cliquer sur le bouton bleu **Créer un nouvel exécuteur**, et copier le champ "registration token" qui s'affiche.

On retourne ensuite dans le terminal, et on entre : 

```bash
# Enregistrement du runner
./act_runner register
```

On entre ensuite simplement le token copié plus tôt, et le tour est joué !

*(Source : [Gitea Documentation](https://docs.gitea.com/usage/actions/act-runner))*

### Utilisation des Gitea Actions

Les Gitea Actions fonctionnent de manière assez similaire à celles de GitHub ou à Gitlab CI, c'est-à-dire avec un fichier `.yml`, dont voici un exemple :

```yml
name: Gitea Actions Demo
run-name: ${{ gitea.actor }} is testing out Gitea Actions
on: [push]

jobs:
  Explore-Gitea-Actions:
    runs-on: ubuntu-latest
    steps:
      - run: echo "The job was automatically triggered by a ${{ gitea.event_name }} event."
      - run: echo "This job is now running on a ${{ runner.os }} server hosted by Gitea!"
      - run: echo "The name of your branch is ${{ gitea.ref }} and your repository is ${{ gitea.repository }}."
      - name: Check out repository code
        uses: actions/checkout@v4
      - run: echo "The ${{ gitea.repository }} repository has been cloned to the runner."
      - run: echo "The workflow is now ready to test your code on the runner."
      - name: List files in the repository
        run: |
          ls ${{ gitea.workspace }}
      - run: echo "This job's status is ${{ job.status }}."
```

À l'instar de GitHub, il suffit de placer ce fichier dans `.gitea/workflows/exemple.yml`, et les changements seront **automatiquement pris en compte**.

Le workflow sera ainsi exécuté selon le fichier YML à chaque push.

*(Source : [Gitea Documentation](https://docs.gitea.com/usage/actions/quickstart))*

