# Procédure d'installation 

Voici la procédure d'installation utilisée pour la création de la machine.

## Choix du système d'exploitation

Cette machine virtuelle utilise Debian. Il s'agit d'une distribution Linux crée en **1991** par Ian Murdock. *(Selon la page [À Propos de Debian](https://www.debian.org/intro/about.fr.html))*

![Logo Debian](assets/openlogo-100.jpg)

> Son nom vient en fait du nom de la femme de Ian, Deborah, ainsi que de son propre nom, eux deux étant très impliqué sur le projet. *([Bref historique de Debian](https://www.debian.org/doc/manuals/project-history/project-history.fr.pdf))*

Debian possède l'avantage d'être encore **une des seules distributions GNU/Linux connues complètement open-source et sous license libre, gérées par des organisations plutôt que des entreprises**. En effet, la plupart des autres choix *mainstream* sont aujourd'hui gérés par des entreprises, à l'instar d'Ubuntu (Canonical).  
Debian tient à sa liberté, en faisant le choix à chaque fois de **n'inclure que des logiciels libres par défaut dans la distrubution**.

## Sélection de l'image disque

La première étape consistait ici à récupérer l'image disque (.iso) sur le site officiel du projet Debian.

Il s'agit d'un fichier représentant un disque virtuel (un ensemble de données étant agencées selon les spécifications de tables de partitionnement (GPT, DOS) et de systèmes de fichiers conventionnels comme FAT32, Ext4, ou encore NTFS. *([Ionos.fr](https://www.ionos.fr/digitalguide/serveur/know-how/quest-ce-quun-fichier-iso/))*

Ce type de fichier a plusieurs avantages : 
 - Peut être gravé sur un média amovible (ou non)
 - Peut être utilisé comme disque virtuel pour une machine virtuelle. 

Il s'agit donc d'un format de fichier optimal pour la distribution de systèmes d'exploitation comme Debian.

> **Attention**  
> La machine hôte étant ici équipée d'un processeur ARM (Apple Silicon), on utilisera une image disque Debian ARM.  
> Cependant, la procédure reste exactement la même avec une image x86_64 (amd64)

> **Bref historique**  
> Depuis 2010, le passage a été progressivement fait vers des machines avec jeux d'instruction 64-bit. Ils sont bien plus efficaces que des processeurs 32-bit, car ils permettent essentiellement d'adresser 2^(64) (16EiO) octets de mémoire centrale, contre seulement 2^(32) (4 GiO) pour les processeurs 32-bit. On peut donc gérer plus de RAM, mais aussi des instructions plus longues.

Le choix a été fait d'utiliser une image `netinst`, qui reste assez légère (~300MB) et avantage les bonnes connexions internet.

Cette image permet d'**installer exactement ce dont l'utilisateur aura besoin**, sans programme additionnel potentiellement non-voulu, à l'inverse d'une image `live-CD`, qui **contient directement le contenu du système d'exploitation dans l'image iso**. *([Documentation Debian](https://www.debian.org/releases/stable/i386/ch04s01.fr.html))*

## Création de la machine virtuelle

Ici, l'application de virtualisation utilisée est [Oracle VirtualBox](https://www.virtualbox.org). Il s'agit d'un programme open-source de virtualisation, ici (sous macOS et Linux) se basant sur l'hyperviseur Qemu/KVM pour alimenter les machines virtuelles.

Voici les caractéristiques de la machine virtuelle crée : 
 - **Type** : Linux
 - **Version** : Debian (64-bit)
 - **Mémoire vive** : 2048 Mo (2Go)
 - **Disque Dur** : 20Go

> **Un mot sur la configuration réseau**  
> Ici, la configuration réseau de la machine est laissée par défaut. Il s'agit ici de NAT.  
> La traduction d'adresses réseau (NAT ou Network Address Translation en anglais), permet à la machine virtuelle d'accéder à la même connexion internet que la machine hôte, **sans pour autant obtenir une adresse physique sur le réseau local de la machine**. Ça l'avantage de ne pas avoir à allouer d'adresse réseau, mais cela ne permet pas aux autres machines du réseau d'accéder aux services de la machine virtuelle. *([IT-Connect.fr](https://www.it-connect.fr/comprendre-les-differents-types-de-reseaux-virtualbox/#A_Le_mode_NAT))*

*Toute cette configuration est stockée dans un fichier XML géré directement par VirtualBox portant le nom de la machine virtuelle à l'extension `.vbox`*  

*Il est possible de modifier directement ce fichier. Par exemple, on peut changer le nombre de processeurs virtuels de la machine à 2, dans la section `<Platform>` à la fin du fichier :*  
```xml
<Platform architecture="ARM">
    <RTC localOrUTC="UTC"/>
    <Chipset type="ARMv8Virtual"/>
    <CPU count="2"/>
</Platform>
```

## Installation de la machine

L'installation de Debian étant en grande partie guidée, on ne s'attardera pas vraiment sur la procédure en elle-même. La configuration reste basique : 
 - **Hostname** : serveur / pas de domaine
 - **Pays/Langue** : France
 - **Mirroir principal** : `http://debian.polytech-lille.fr` *(C'est le mirroir le plus proche géographiquement, donc potentiellement le plus rapide)*
 - **Proxy** : Aucun
 - **Partitionnement** : Tout dans la même seule partition
 - **Logiciels de démarrage** : 
    - Serveur web
    - Serveur ssh
    - Utilitaires usuels du système
    - *Et donc pas d'environnement de bureau/graphique*

> **Pourquoi pas d'environnement graphique ?**  
> Les environnements de bureau tels que MATE ou GNOME (**GN**U **N**etwork **O**bject **M**odel **E**nvironment) sont un ensemble d'applications et de programmes informatiques *graphiques* permettant l'utilisation d'une machine de manière plus conviviale qu'une *interface texte* comme un terminal. *([Wiki Debian](https://wiki.debian.org/fr/DesktopEnvironment))*  
> Ici, on préfère éviter d'en utiliser un, pour des raisons de légèreté. Le serveur X11 utilisé pour faire tourner les applications graphiques étant très consommateur en mémoire, on préfère économiser cette mémoire vive pour pouvoir la dédier aux services tournant sur la machine. On peut toujours accéder à la machine via la console `tty` ou par SSH. *([Ubuntu Wiki](https://help.ubuntu.com/community/ServerGUI))*

> **Un serveur SSH**  
> Il s'agit d'un service (Secure Shell) permettant d'exécuter des commandes sur la machine via une machine distante, présente dans le même réseau ou non (donc via un VPN). 

> **Un serveur web**  
> Un serveur web permettra l'affichage d'une page web, en répondant aux requêtes `HTTP` (HyperText Transport Protocol) (souvent sur le port 80 ou 8080) par des pages présentes localement sur la machine.

> **À propos de la configuration d'un proxy**  
> Ici, on ne configurera pas de serveur mandataire (ou proxy). Il s'agit d'un service faisant office d'intermédiaire entre deux hôtes pour faciliter ou surveiller leurs échanges (par exemple filtrer et empêcher des requêtes DNS).  
> Ici, on n'en aura pas besoin, car aucun proxy n'est installé sur la machine hôte. S'il y en a un sur le réseau, la machine hôte a déjà été configurée pour accéder à internet à travers le proxy. Le NAT de VirtualBox permet d'éviter de tout reconfigurer.

Une fois l'installation de la machine terminée, on peut passer à sa configuration.

## Configuration de la machine

### Accès root pour `user`

L'utilisateur crée par l'installation de Debian ne possède pour l'instant pas d'accès aux permissions `root` et ne peut donc pour l'instant que modifier ses propres fichiers (essentiellement son dossier dans le `/home`).

Une solution possible est l'utilisation du programme `sudo`. Il agit à la manière d'un intermédiaire que l'on configure pour exécuter certaines actions en tant que superutilisateur.

> **Pourquoi ne pas directement accéder à la machine en tant que `root` ?**  
> Il est préférable d'éviter d'accéder à la machine en tant que `root` pour des raisons de sécurité. L'accès distant (SSH) au serveur pouvant être obtenu sans autorisation, on préfère prévenir les dégâts trop importants en limitant une attaque à un seul utilisateur. Aussi, certaines actions ne sont faisables qu'en tant qu'utilisateur normal, comme des tâches de compilation de code ou autre, pour éviter de causer des dégâts importants à la machine. *([Wiki Ubuntu](https://doc.ubuntu-fr.org/tutoriel/comment_eviter_root_quand_c_est_possible)*) 

Le programme `sudo` est installé par défaut sur Debian si on a accepté de créer un compte utilisateur. On peut donc activer les droits pour `user` avec la commande (en tant que `root` pour commencer) : 

```bash
usermod -aG sudo user
```

`sudo` est configuré par défaut pour permettre aux utilisateurs membres du groupe `sudo` d'accéder aux droits complets d'administrateur, via la commande :
```
sudo <commande>
```

On peut vérifier son bon déroulement en affichant les groupes auxquels `user` appartient : 
```bash
groups user
``` 
Ou sans argument si on est connecté directement en tant que `user`.

### Installation des VBox GAs

Les VBox Guest Additions (dont on évitera de traduire le nom en français pour des raisons évidentes) est un module pour le noyau Linux permettant de faciliter l'interaction entre la machine hôte et la machine virtuelle, comme par exemple : 
 - La mise en place de dossiers partagés entre l'hôte et l'invité
 - Le partage du presse-papier permettant de copier-coller aussi bien sur la machine virtuelle que sur l'hôte
 - Le redimensionnement automatique de l'écran selon la taille de la fenêtre

Pour l'installer, il faut d'abord s'assurer de disposer des outils permettant de compiler le module. 

#### Installation des outils de compilation. 

Sur Debian, comme sur beaucoup d'autres distributions Linux, les outils de compilation (GCC, Make, etc) ne sont pas directement installés sur le système, afin d'économiser de l'espace.  
Les bibliothèques partagées nécessaires à la compilation de certains programmes sont également séparées (et c'est une spécificité des distributions de la famille de Debian) dans les paquets (*le paquet ncurses ne contient que la fonctionnalité nécessaire au fonctionnement des programmes basés sur `ncurses`, mais pas leur compilation, dont les outils seront contenus dans `ncurses-dev`*)

Il faudra donc les installer sur le système via la commande suivante : 
```bash 
sudo apt install build-essential
```

Il faut également installer les `headers` du noyau Linux, qui sont **obligatoires** à la compilation de modules pour le noyau : 
```bash
sudo apt install linux-headers-generic
```

> **Versions du kernel et headers**  
> Sur Debian, plusieurs versions du noyau Linux sont disponibles dans les dépôts. En fonction de la version de l'image ISO choisie, cette version peut changer. Il y a également des versions spéciales pour certaines configurations particulières, comme une image allégée pour les machines virtuelles, ou pour les systèmes embarqués.  
> Sur Debian 12, la version du Kernel (noyau) Linux est la 6.1. Il est founri par défaut par le paquet `linux-image-generic`. On installe donc les headers `linux-headers-generic` liés au paquet.

#### Compilation du module 

L'étape suivante est la compilation du module après l'insertion de l'image disque de VirtualBox.

Dans la barre de menus de la fenêtre, on ira dans **Périphériques > Insérer l'image disque des Additions Invités**. Le CD-Rom contenant le logiciel devrait être inséré dans la machine.

Une fois le disque inséré, on doit récupérer son *block device name*. C'est souvent un nom en `srX` (sûrement `sr0`) :

```bash
lsblk
```

On monte ensuite le CD-ROM sur le dossier `/mnt`
```bash
# Remplacer srX par le block device name du CD-ROM
mount /dev/srX /mnt
```

Cette commande permet d'afficher le contenu du CD-ROM dans le dossier `/mnt` spécialement pourvu pour cet usage sur Linux.

Il est probable qu'un message indiquant que le disque est protégé en écriure s'affiche. Ce n'est pas important.

On exécute ensuite le fichier permettant la compilation et l'installation du module : 

```bash
sudo /mnt/VBoxLinuxAdditions.run
```

Personnellement, j'exécute la version optimisée pour les processeurs ARM sur mon ordinateur, afin d'optimiser ses performances. Son nom est `VBoxLinuxAdditions-arm64.run`.

Une fois toutes ces étapes franchies, on est prêt à passer à la suite. 